@@ -1,8 +1,8 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2004 Peter Selinger.
+# Copyright (C) 2001-2005 Peter Selinger.
 # This file is part of potrace. It is free software and it is covered
-# by the GNU general public license. See the file COPYING for details.
+# by the GNU General Public License. See the file COPYING for details.
 
 echo "Checking input formats..." >& 2
 
@@ -17,7 +17,7 @@ fi
 
 NAME=`basename "$0"`
 
-POTRACE="../src/potrace"
+POTRACE="../src/potrace --progress"
 TMPDIR="${TEMPDIR:-/tmp}"
 TMP1=`mktemp "$TMPDIR/$NAME-1.XXXXXX"`
 TMP2=`mktemp "$TMPDIR/$NAME-2.XXXXXX"`
@@ -40,10 +40,10 @@ INFILES="data1.pbm data1.pgm data1.ppm data1.bmp1 data1.bmp4 data1.bmp8 data1.bm
 # extract first file
 set dummy $INFILES; F="$2"
 
-action "$POTRACE" -o "$TMP1" "$srcdir/$F"
+action $POTRACE -o "$TMP1" "$srcdir/$F"
 
 for i in $INFILES; do
-    action "$POTRACE" -o "$TMP2" "$srcdir/$i"
+    action $POTRACE -o "$TMP2" "$srcdir/$i"
     action diff "$TMP1" "$TMP2" > /dev/null
     action rm -f "$TMP2"
 done
