@@ -1,8 +1,8 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2004 Peter Selinger.
+# Copyright (C) 2001-2005 Peter Selinger.
 # This file is part of potrace. It is free software and it is covered
-# by the GNU general public license. See the file COPYING for details.
+# by the GNU General Public License. See the file COPYING for details.
 
 echo "Checking mkbitmap..." >& 2
 
@@ -53,10 +53,10 @@ actiondiff () {
 alias action="LINE=\$LINENO; action"
 alias actiondiff="LINE=\$LINENO; actiondiff"
 
-action "$MKBITMAP" -f2 -s2 -t.48 -o "$TMP1" "$DATA"
+action $MKBITMAP -f2 -s2 -t.48 -o "$TMP1" "$DATA"
 actiondiff "$TMP1" "$REFDATA1" 50
 
-action "$MKBITMAP" -f1 -s2 -t.54 -1 -i -o "$TMP1" "$DATA"
+action $MKBITMAP -f1 -s2 -t.54 -1 -i -o "$TMP1" "$DATA"
 actiondiff "$TMP1" "$REFDATA2" 50
 
 action rm -f "$TMP1"
