@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_eps.c,v 1.14 2004/01/15 05:07:22 selinger Exp $ */
+/* $Id: backend_eps.c,v 1.23 2005/02/22 21:31:23 selinger Exp $ */
 
 /* The Postscript backend of potrace. This can produce "ps" or "eps"
    output, and different kinds of graphical debugging
@@ -15,19 +15,18 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include "potracelib.h"
+#include "curve.h"
 #include "main.h"
 #include "backend_eps.h"
 #include "flate.h"
 #include "lists.h"
-#include "curve.h"
+#include "auxiliary.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#define max(a,b) ((a)>(b) ? (a) : (b))
-#define abs(a) ((a)>0 ? (a) : -(a))
-
 #define SAFE_MALLOC(var, n, typ) \
   if ((var = (typ *)malloc((n)*sizeof(typ))) == NULL) goto malloc_error 
 
@@ -96,7 +95,7 @@ static void eps_callbacks(FILE *fout) {
 /* postscript path-drawing auxiliary functions */
 
 /* coordinate quantization */
-static point_t unit(dpoint_t p) {
+static inline point_t unit(dpoint_t p) {
   point_t q;
 
   q.x = (long)(floor(p.x*info.unit+.5));
@@ -201,21 +200,22 @@ static void eps_linewidth(double w) {
 /* ASCII encoding */
 
 /* explicit encoding, does not use special macros */
-static int eps_path_long(curve_t *curve, int m) {
+static int eps_path_long(privcurve_t *curve) {
   int i;
   dpoint_t *c;
+  int m = curve->n;
 
-  c = curve[m-1].c;
+  c = curve->c[m-1];
   eps_moveto(c[2]);
 
   for (i=0; i<m; i++) {
-    c = curve[i].c;
-    switch (curve[i].tag) {
-    case CORNER:
+    c = curve->c[i];
+    switch (curve->tag[i]) {
+    case POTRACE_CORNER:
       eps_lineto(c[1]);
       eps_lineto(c[2]);
       break;
-    case CURVETO:
+    case POTRACE_CURVETO:
       eps_curveto(c[0], c[1], c[2]);
       break;
     }
@@ -224,13 +224,14 @@ static int eps_path_long(curve_t *curve, int m) {
 }
 
 /* size-optimized encoding relies on special macros */
-static int eps_path_short(curve_t *curve, int m) {
+static int eps_path_short(privcurve_t *curve) {
   int i, i1;
   long int *bq = NULL;  /* bq[m] */
   long int *aq = NULL;  /* aq[m] */
   point_t *v = NULL;    /* v[m] */
   dpoint_t *q = NULL;   /* q[m] */
   double M;
+  int m = curve->n;
 
   SAFE_MALLOC(bq, m, long int);
   SAFE_MALLOC(aq, m, long int);
@@ -239,15 +240,15 @@ static int eps_path_short(curve_t *curve, int m) {
 
   /* quantize vertices */
   for (i=0; i<m; i++) {
-    v[i] = unit(curve[i].vertex);
+    v[i] = unit(curve->vertex[i]);
   }
 
   /* quantize beta */
   for (i=0; i<m; i++) {
     i1 = mod(i+1,m);
     M = max(10, max(abs(v[i1].x-v[i].x), abs(v[i1].y-v[i].y)));
-    bq[i] = (int)(M * curve[i].beta + 0.5);
-    if (curve[i].beta != 0.5) {
+    bq[i] = (int)(M * curve->beta[i] + 0.5);
+    if (curve->beta[i] != 0.5) {
       q[i1] = interval(bq[i]/M, dpoint(v[i]), dpoint(v[i1]));
     } else {
       q[i1] = interval(0.5, dpoint(v[i]), dpoint(v[i1]));
@@ -259,8 +260,8 @@ static int eps_path_short(curve_t *curve, int m) {
     i1 = mod(i+1,m);
     M = max(10, max(max(abs(q[i].x-v[i].x), abs(q[i].y-v[i].y)),
 		    max(abs(v[i].x-q[i1].x), abs(v[i].y-q[i1].y))));
-    if (curve[i].tag == CURVETO) {
-      aq[i] = (int)(M * curve[i].alpha + 0.5);
+    if (curve->tag[i] == POTRACE_CURVETO) {
+      aq[i] = (int)(M * curve->alpha[i] + 0.5);
       if (aq[i] > M) {
 	aq[i]--;
       }
@@ -270,7 +271,7 @@ static int eps_path_short(curve_t *curve, int m) {
   /* generate output */
   ship("%ld %ld ", v[m-1].x, v[m-1].y);
   ship("%ld %ld ", v[0].x - v[m-1].x, v[0].y - v[m-1].y);
-  if (curve[m-1].beta == 0.5) {
+  if (curve->beta[m-1] == 0.5) {
     ship("i\n");
   } else {
     ship("%ld I\n", bq[m-1]);
@@ -278,14 +279,14 @@ static int eps_path_short(curve_t *curve, int m) {
   for (i=0; i<m; i++) {
     if (i<m-1) {
       ship("%ld %ld ", v[i+1].x - v[i].x, v[i+1].y - v[i].y);
-      if (curve[i].beta != 0.5) {
+      if (curve->beta[i] != 0.5) {
 	ship("%ld ", bq[i]);
       }
     }
-    if (curve[i].tag == CURVETO) {
-      ship(curve[i].beta == 0.5 ? "%ld c\n" : "%ld C\n", aq[i]);
+    if (curve->tag[i] == POTRACE_CURVETO) {
+      ship(curve->beta[i] == 0.5 ? "%ld c\n" : "%ld C\n", aq[i]);
     } else {
-      ship(curve[i].beta == 0.5 ? "v\n" : "V\n");
+      ship(curve->beta[i] == 0.5 ? "v\n" : "V\n");
     }
   }  
 
@@ -303,11 +304,11 @@ static int eps_path_short(curve_t *curve, int m) {
   return 1;
 }
 
-static int eps_path(curve_t *curve, int m) {
+static int eps_path(privcurve_t *curve) {
   if (info.longcoding) {
-    return eps_path_long(curve, m);
+    return eps_path_long(curve);
   } else {
-    return eps_path_short(curve, m);
+    return eps_path_short(curve);
   }
 }
 
@@ -316,30 +317,30 @@ static int eps_path(curve_t *curve, int m) {
    generate debugging output */
 
 /* output jaggie curve in grey */
-static void eps_jaggy(path_t *plist) {
-  path_t *p;
+static void eps_jaggy(potrace_path_t *plist) {
+  potrace_path_t *p;
   int i;
 
   ship(".9 setgray\n");
   list_forall (p, plist) {
-    point_t *pt = p->pt;
+    point_t *pt = p->priv->pt;
     point_t cur, prev;
 
     if (p->sign == '+') {
-      cur = prev = pt[p->len-1];
+      cur = prev = pt[p->priv->len-1];
       eps_moveto(dpoint(cur));
-      for (i=0; i<p->len; i++) {
+      for (i=0; i<p->priv->len; i++) {
 	if (pt[i].x != cur.x && pt[i].y != cur.y) {
 	  cur = prev;
 	  eps_lineto(dpoint(cur));
 	}
 	prev = pt[i];
       }
-      eps_lineto(dpoint(pt[p->len-1]));
+      eps_lineto(dpoint(pt[p->priv->len-1]));
     } else {
       cur = prev = pt[0];
       eps_moveto(dpoint(cur));
-      for (i=p->len-1; i>=0; i--) {
+      for (i=p->priv->len-1; i>=0; i--) {
 	if (pt[i].x != cur.x && pt[i].y != cur.y) {
           cur = prev;
           eps_lineto(dpoint(cur));
@@ -355,39 +356,41 @@ static void eps_jaggy(path_t *plist) {
 }
 
 /* output polygon */
-void eps_polygon(curve_t *curve, int m, const color_t col) {
+static void eps_polygon(privcurve_t *curve, const color_t col) {
   int i;
+  int m = curve->n;
 
   eps_linewidth(.02);
   eps_setcolor(col);
-  eps_moveto(curve[m-1].vertex);
+  eps_moveto(curve->vertex[m-1]);
   for (i=0; i<m; i++) {
-    eps_lineto(curve[i].vertex);
+    eps_lineto(curve->vertex[i]);
   }
   ship("stroke\n");
 }
 
 /* output lines L and parameter alpha */
-void eps_L(curve_t *curve, int m, const color_t col) {
+static void eps_L(privcurve_t *curve, const color_t col) {
   int i, i1;
   double gamma;
   dpoint_t p1, p4, p1l, p4l;
-  
+  int m = curve->n;
+ 
   for (i=0; i<m; i++) {
     i1 = mod(i+1, m);
-    gamma = curve[i1].alpha0 * 0.75;
+    gamma = curve->alpha0[i1] * 0.75;
     
-    p1 = curve[i].c[2];
-    p4 = curve[i1].c[2];
-    p1l = interval(gamma, p1, curve[i1].vertex);
-    p4l = interval(gamma, p4, curve[i1].vertex);
+    p1 = curve->c[i][2];
+    p4 = curve->c[i1][2];
+    p1l = interval(gamma, p1, curve->vertex[i1]);
+    p4l = interval(gamma, p4, curve->vertex[i1]);
     eps_linewidth(.02);
     eps_setcolor(col);
     eps_moveto(p1l);
     eps_lineto(p4l);
     ship("stroke\n");
-    eps_moveto_offs(curve[i1].vertex, -.4, -.4);
-    ship("times (%.2f) show\n", curve[i1].alpha0);
+    eps_moveto_offs(curve->vertex[i1], -.4, -.4);
+    ship("times (%.2f) show\n", curve->alpha0[i1]);
   }
 }
 
@@ -441,13 +444,13 @@ static char *debugmacros =
 /* Backends for various types of output. */
 
 /* Normal output: black on transparent */
-static int render0(path_t *plist) {
-  path_t *p;
+static int render0(potrace_path_t *plist) {
+  potrace_path_t *p;
 
   if (info.longcoding) {
     eps_setcolor(info.color);
     list_forall (p, plist) {
-      eps_path(p->fcurve, p->fm);
+      eps_path(p->priv->fcurve);
       ship("closepath\n");
       if (p->next == NULL || p->next->sign == '+') {
 	ship("fill\n");
@@ -455,7 +458,7 @@ static int render0(path_t *plist) {
     }
   } else {
     list_forall (p, plist) {
-      eps_path(p->fcurve, p->fm);
+      eps_path(p->priv->fcurve);
       if (p->next == NULL || p->next->sign == '+') {
 	ship("b\n");
       } else {
@@ -467,19 +470,19 @@ static int render0(path_t *plist) {
 }
 
 /* Opaque output: alternating black and white */
-static int render0_opaque(path_t *plist) {
-  path_t *p;
+static int render0_opaque(potrace_path_t *plist) {
+  potrace_path_t *p;
   
   if (info.longcoding) {
     list_forall (p, plist) {
-      eps_path(p->fcurve, p->fm);
+      eps_path(p->priv->fcurve);
       ship("closepath\n");
       eps_setcolor(p->sign=='+' ? info.color : info.fillcolor);
       ship("fill\n");
     }
   } else {
     list_forall (p, plist) {
-      eps_path(p->fcurve, p->fm);
+      eps_path(p->priv->fcurve);
       ship(p->sign=='+' ? "b\n" : "w\n");
     }
   }
@@ -487,18 +490,18 @@ static int render0_opaque(path_t *plist) {
 }
 
 /* Debug output type 1 (show optimal polygon) */
-static int render1(path_t *plist) {
-  path_t *p;
+static int render1(potrace_path_t *plist) {
+  potrace_path_t *p;
   int i;
 
   eps_jaggy(plist);
 
   list_forall (p, plist) {
 
-    point_t *pt = p->pt;
-    int n = p->len;
-    int m = p->m;
-    int *po = p->po;
+    point_t *pt = p->priv->pt;
+    int n = p->priv->len;
+    int m = p->priv->m;
+    int *po = p->priv->po;
 
     eps_linewidth(.02);
     eps_setcolor(black);
@@ -534,8 +537,8 @@ static int render1(path_t *plist) {
 }
 
 /* Debug output type 2 (show corrected polygon and edge detection) */
-static int render2(path_t *plist) {
-  path_t *p;
+static int render2(potrace_path_t *plist) {
+  potrace_path_t *p;
   int i;
 
   /* output original bitmap in grey */
@@ -543,34 +546,34 @@ static int render2(path_t *plist) {
   
   list_forall (p, plist) {
     /* output polygon with corrected edges, lines L, and parameter alpha */
-    eps_polygon(p->curve, p->m, black);
-    eps_L(p->curve, p->m, black);
+    eps_polygon(&p->priv->curve, black);
+    eps_L(&p->priv->curve, black);
     
     /* output the vertex unit squares */
-    for (i=0; i<p->m; i++) {
-      eps_moveto(p->curve[i].vertex);
+    for (i=0; i<p->priv->curve.n; i++) {
+      eps_moveto(p->priv->curve.vertex[i]);
       ship("usq\n");
     }
 
     /* output the path */
     eps_linewidth(.1);
     eps_setcolor(blue);
-    eps_path(p->curve, p->m);
+    eps_path(&p->priv->curve);
     ship("closepath\n");
     ship("stroke\n");
 
-    if (info.opticurve && info.debug == 3) {
+    if (info.param->opticurve && info.debug == 3) {
 
       /* output opticurve */
       eps_linewidth(.05);
       eps_setcolor(red);
-      eps_path(p->ocurve, p->om);
+      eps_path(&p->priv->ocurve);
       ship("closepath\n");
       ship("stroke\n");
       
       /* output dots */
-      for (i=0; i<p->om; i++) {
-	eps_coords(p->ocurve[i].c[2]);
+      for (i=0; i<p->priv->ocurve.n; i++) {
+	eps_coords(p->priv->ocurve.c[i][2]);
 	ship("dot1\n");
       }
     }
@@ -579,8 +582,8 @@ static int render2(path_t *plist) {
 }
 
 /* Free-style debug output */
-static int render_debug(path_t *plist) {
-  path_t *p;
+static int render_debug(potrace_path_t *plist) {
+  potrace_path_t *p;
   int count;
   int i;
 
@@ -592,54 +595,54 @@ static int render_debug(path_t *plist) {
     count++;
 
     /* output path numbers */
-    eps_moveto_offs(p->curve[0].vertex, 0, 5);
+    eps_moveto_offs(p->priv->curve.vertex[0], 0, 5);
     ship("times1 (%d) show\n", count);
 
     /* output polygon with corrected edges, lines L, and parameter alpha */
-    eps_polygon(p->curve, p->m, black);
-    eps_L(p->curve, p->m, black);
+    eps_polygon(&p->priv->curve, black);
+    eps_L(&p->priv->curve, black);
 
     /* output the vertex unit squares */
-    for (i=0; i<p->m; i++) {
-      eps_moveto(p->curve[i].vertex);
+    for (i=0; i<p->priv->curve.n; i++) {
+      eps_moveto(p->priv->curve.vertex[i]);
       ship("usq\n");
     }
 
     /* output the vertex numbers */
-    for (i=0; i<p->m; i++) {
-      eps_moveto_offs(p->curve[i].vertex, +1, +1);
+    for (i=0; i<p->priv->curve.n; i++) {
+      eps_moveto_offs(p->priv->curve.vertex[i], +1, +1);
       ship("times2 (%d) show\n", i);
     }
     
     /* output the path */
     eps_linewidth(.1);
     eps_setcolor(blue);
-    eps_path(p->curve, p->m);
+    eps_path(&p->priv->curve);
     ship("closepath\n");
     ship("stroke\n");
     
-    if (info.opticurve) {
+    if (info.param->opticurve) {
 
       /* output the opti-verteces polygon */
-      eps_polygon(p->ocurve, p->om, green);
+      eps_polygon(&p->priv->ocurve, green);
       
       /* output opticurve */
       eps_linewidth(.05);
       eps_setcolor(red);
-      eps_path(p->ocurve, p->om);
+      eps_path(&p->priv->ocurve);
       ship("closepath\n");
       ship("stroke\n");
       
       /* output dots */
-      for (i=0; i<p->om; i++) {
-	eps_coords(p->ocurve[i].c[2]);
+      for (i=0; i<p->priv->ocurve.n; i++) {
+	eps_coords(p->priv->ocurve.c[i][2]);
 	ship("dot1\n");
       }
 
       /* output beta parameters */
-      for (i=0; i<p->om; i++) {
-	eps_moveto_offs(p->ocurve[i].c[2], +.4, -.4);
-	ship("times (%.2f) show\n", p->ocurve[i].beta);
+      for (i=0; i<p->priv->ocurve.n; i++) {
+	eps_moveto_offs(p->priv->ocurve.c[i][2], +.4, -.4);
+	ship("times (%.2f) show\n", p->priv->ocurve.beta[i]);
       }
     }
   }
@@ -647,7 +650,7 @@ static int render_debug(path_t *plist) {
 }
 
 /* select the appropriate rendering function from above */
-static int eps_render(path_t *plist) {
+static int eps_render(potrace_path_t *plist) {
   int r;
   
   switch (info.debug) {
@@ -682,7 +685,7 @@ static int eps_init(imginfo_t *imginfo) {
   char *c0, *c1;
 
   shipcom("%%!PS-Adobe-3.0 EPSF-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2004\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2005\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
   shipcom("%%%%BoundingBox: 0 0 %d %d\n", 
 	  (int)ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar),
@@ -720,7 +723,7 @@ static int eps_term() {
 }
 
 /* public interface for EPS */
-int page_eps(FILE *fout, path_t *plist, imginfo_t *imginfo) {
+int page_eps(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   int r;
 
   eps_callbacks(fout);
@@ -749,7 +752,7 @@ int init_ps(FILE *fout) {
   eps_callbacks(fout);
 
   shipcom("%%!PS-Adobe-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2004\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2005\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
   shipcom("%%%%BoundingBox: 0 0 %d %d\n", info.paperwidth, info.paperheight);
   shipcom("%%%%Pages: (atend)\n");
@@ -810,7 +813,7 @@ static void eps_pageterm_ps() {
   ship("showpage\n");
 }
 
-int page_ps(FILE *fout, path_t *plist, imginfo_t *imginfo) {
+int page_ps(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
   int r;
 
   eps_callbacks(fout);
