@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_svg.c,v 1.6 2004/01/15 05:07:22 selinger Exp $ */
+/* $Id: backend_svg.c,v 1.14 2005/02/22 21:31:23 selinger Exp $ */
 
 /* The SVG backend of potrace. */
 
@@ -11,10 +11,12 @@
 #include <string.h>
 #include <math.h>
 
+#include "potracelib.h"
+#include "curve.h"
 #include "main.h"
 #include "backend_svg.h"
-#include "curve.h"
 #include "lists.h"
+#include "auxiliary.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -24,7 +26,7 @@
 /* path-drawing auxiliary functions */
 
 /* coordinate quantization */
-static point_t unit(dpoint_t p) {
+static inline point_t unit(dpoint_t p) {
   point_t q;
 
   q.x = (long)(floor(p.x*info.unit+.5));
@@ -125,11 +127,12 @@ static void svg_curveto(FILE *fout, dpoint_t p1, dpoint_t p2, dpoint_t p3) {
 
 /* Explicit encoding. If abs is set, move to first coordinate
    absolutely. */
-static int svg_path(FILE *fout, curve_t *curve, int m, int abs) {
+static int svg_path(FILE *fout, potrace_curve_t *curve, int abs) {
   int i;
   dpoint_t *c;
+  int m = curve->n;
 
-  c = curve[m-1].c;
+  c = curve->c[m-1];
   if (abs) {
     svg_moveto(fout, c[2]);
   } else {
@@ -137,13 +140,13 @@ static int svg_path(FILE *fout, curve_t *curve, int m, int abs) {
   }
 
   for (i=0; i<m; i++) {
-    c = curve[i].c;
-    switch (curve[i].tag) {
-    case CORNER:
+    c = curve->c[i];
+    switch (curve->tag[i]) {
+    case POTRACE_CORNER:
       svg_lineto(fout, c[1]);
       svg_lineto(fout, c[2]);
       break;
-    case CURVETO:
+    case POTRACE_CURVETO:
       svg_curveto(fout, c[0], c[1], c[2]);
       break;
     }
@@ -188,8 +191,8 @@ static int svg_jaggy_path(FILE *fout, point_t *pt, int n, int abs) {
   return 0;
 }
 
-void write_paths_opaque(FILE *fout, path_t *tree) {
-  path_t *p, *q;
+static void write_paths_opaque(FILE *fout, potrace_path_t *tree) {
+  potrace_path_t *p, *q;
   int c;
 
   for (p=tree; p; p=p->sibling) {
@@ -202,9 +205,9 @@ void write_paths_opaque(FILE *fout, path_t *tree) {
     newline = 1;
     lastop = 0;
     if (info.debug == 1) {
-      svg_jaggy_path(fout, p->pt, p->len, 1);
+      svg_jaggy_path(fout, p->priv->pt, p->priv->len, 1);
     } else {
-      svg_path(fout, p->fcurve, p->fm, 1);
+      svg_path(fout, &p->curve, 1);
     }
     fprintf(fout, "\"/>\n");
     for (q=p->childlist; q; q=q->sibling) {
@@ -213,9 +216,9 @@ void write_paths_opaque(FILE *fout, path_t *tree) {
       newline = 1;
       lastop = 0;
       if (info.debug == 1) {
-	svg_jaggy_path(fout, q->pt, q->len, 1);
+	svg_jaggy_path(fout, q->priv->pt, q->priv->len, 1);
       } else {
-	svg_path(fout, q->fcurve, q->fm, 1);
+	svg_path(fout, &q->curve, 1);
       }
       fprintf(fout, "\"/>\n");
     }
@@ -231,8 +234,8 @@ void write_paths_opaque(FILE *fout, path_t *tree) {
   }
 }
 
-void write_paths_transparent(FILE *fout, path_t *tree) {
-  path_t *p, *q;
+static void write_paths_transparent(FILE *fout, potrace_path_t *tree) {
+  potrace_path_t *p, *q;
   int c;
 
   for (p=tree; p; p=p->sibling) {
@@ -244,15 +247,15 @@ void write_paths_transparent(FILE *fout, path_t *tree) {
     newline = 1;
     lastop = 0;
     if (info.debug == 1) {
-      svg_jaggy_path(fout, p->pt, p->len, 1);
+      svg_jaggy_path(fout, p->priv->pt, p->priv->len, 1);
     } else {
-      svg_path(fout, p->fcurve, p->fm, 1);
+      svg_path(fout, &p->curve, 1);
     }
     for (q=p->childlist; q; q=q->sibling) {
       if (info.debug == 1) {
-	svg_jaggy_path(fout, q->pt, q->len, 0);
+	svg_jaggy_path(fout, q->priv->pt, q->priv->len, 0);
       } else {
-	svg_path(fout, q->fcurve, q->fm, 0);
+	svg_path(fout, &q->curve, 0);
       }
     }
     fprintf(fout, "\"/>\n");
@@ -269,7 +272,7 @@ void write_paths_transparent(FILE *fout, path_t *tree) {
 /* Backend. */
 
 /* public interface for SVG */
-int page_svg(FILE *fout, path_t *plist, imginfo_t *imginfo) {
+int page_svg(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo) {
 
   int bboxx = (int)ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar);
   int bboxy = (int)ceil(imginfo->trans.bb[1]+imginfo->tmar+imginfo->bmar);
@@ -291,7 +294,7 @@ int page_svg(FILE *fout, path_t *plist, imginfo_t *imginfo) {
 
   /* metadata: creator */
   fprintf(fout, "<metadata>\n");
-  fprintf(fout, "Created by "POTRACE" "VERSION", written by Peter Selinger 2001-2004\n");
+  fprintf(fout, "Created by "POTRACE" "VERSION", written by Peter Selinger 2001-2005\n");
   fprintf(fout, "</metadata>\n");
 
   /* use a "group" tag to establish coordinate system and style */
