@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: flate.c,v 1.6 2004/06/20 15:30:30 selinger Exp $ */
+/* $Id: flate.c,v 1.10 2005/02/22 21:31:23 selinger Exp $ */
 
 /* the PostScript compression module of potrace. The basic interface
    is through the *_xship function, which processes a byte array and
@@ -39,6 +39,9 @@
 
 #define OUTSIZE 1000
 
+static int a85init(FILE *f);
+static int a85finish(FILE *f);
+static int a85write(FILE *f, char *buf, int n);
 static int a85out(FILE *f, int n);
 static int a85spool(FILE *f, char c);
 
@@ -55,6 +58,66 @@ int dummy_xship(FILE *f, int filter, char *s, int len) {
 
 #ifdef HAVE_ZLIB
 
+int pdf_xship(FILE *f, int filter, char *s, int len) {
+	static int fstate = 0;
+	static z_stream c_stream;
+	char outbuf[OUTSIZE];
+	int err;
+	int n=0;
+
+  if (filter & !fstate) {
+    /* switch on filtering */
+    c_stream.zalloc = Z_NULL;
+    c_stream.zfree = Z_NULL;
+    c_stream.opaque = Z_NULL;
+    err = deflateInit(&c_stream, 9);
+    if (err != Z_OK) {
+      fprintf(stderr, "deflateInit: %s (%d)\n", c_stream.msg, err);
+      exit(1);
+    }
+    c_stream.avail_in = 0;
+    fstate = 1;
+  } else if (!filter & fstate) {
+    /* switch off filtering */
+    /* flush stream */
+    do {
+      c_stream.next_out = (Bytef*)outbuf;
+      c_stream.avail_out = OUTSIZE;
+
+      err = deflate(&c_stream, Z_FINISH);
+      if (err != Z_OK && err != Z_STREAM_END) {
+	fprintf(stderr, "deflate: %s (%d)\n", c_stream.msg, err);
+	exit(1);
+      }
+      n += fwrite(outbuf, 1, OUTSIZE-c_stream.avail_out, f);
+    } while (err != Z_STREAM_END);
+
+    fstate = 0;
+  }
+  if (!fstate) {
+    fwrite(s, 1, len, f);
+    return n+len;
+  }
+  
+  /* do the actual compression */
+  c_stream.next_in = (Bytef*) s;
+  c_stream.avail_in = len;
+
+  do {
+    c_stream.next_out = (Bytef*) outbuf;
+    c_stream.avail_out = OUTSIZE;
+
+    err = deflate(&c_stream, Z_NO_FLUSH);
+    if (err != Z_OK) {
+      fprintf(stderr, "deflate: %s (%d)\n", c_stream.msg, err);
+      exit(1);
+    }
+    n += fwrite(outbuf, 1, OUTSIZE-c_stream.avail_out, f);
+  } while (!c_stream.avail_out);
+  
+  return n;
+}
+
 /* ship len bytes from s using zlib compression. */
 int flate_xship(FILE *f, int filter, char *s, int len) {
   static int fstate = 0;
@@ -135,7 +198,7 @@ int flate_xship(FILE *f, int filter, char *s, int len) {
 /* use Postscript level 2 compression. Ship len bytes from str. */
 int lzw_xship(FILE *f, int filter, char *str, int len) {
   static int fstate = 0;
-  static lzw_stream *s = NULL;
+  static lzw_stream_t *s = NULL;
   char outbuf[OUTSIZE];
   int err;
   int n=0;
@@ -230,17 +293,17 @@ int a85_xship(FILE *f, int filter, char *s, int len) {
 /* ---------------------------------------------------------------------- */
 /* low-level a85 backend */
 
-unsigned long a85buf[4];
-int a85n;
-int a85col;
+static unsigned long a85buf[4];
+static int a85n;
+static int a85col;
 
-int a85init(FILE *f) {
+static int a85init(FILE *f) {
   a85n = 0;
   a85col = 0;
   return 0;
 }
 
-int a85finish(FILE *f) {
+static int a85finish(FILE *f) {
   int r=0;
 
   if (a85n) {
@@ -250,7 +313,7 @@ int a85finish(FILE *f) {
   return r+2;
 }
 
-int a85write(FILE *f, char *buf, int n) {
+static int a85write(FILE *f, char *buf, int n) {
   int i;
   int r=0;
 
