@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: greymap.c,v 1.5 2004/01/15 05:07:22 selinger Exp $ */
+/* $Id: greymap.c,v 1.7 2005/02/22 21:31:23 selinger Exp $ */
 
 /* Routines for manipulating greymaps, including reading pgm files. We
    only deal with greymaps of depth 8 bits. */
@@ -165,21 +165,14 @@ static int readbit(FILE *f) {
    convert the output to a greymap. Return greymap in *gmp. Return 0
    on success, -1 on error with errno set, -2 on bad file format (with
    error message in gm_read_error), and 1 on premature end of file, -3
-   on empty file, -4 if wrong magic number. If the return value is
-   >=0, *gmp is valid. */
+   on empty file (including files with only whitespace and comments),
+   -4 if wrong magic number. If the return value is >=0, *gmp is
+   valid. */
 
 char *gm_read_error = NULL;
 
 int gm_read(FILE *f, greymap_t **gmp) {
   int magic[2];
-  int c;
-
-  /* check if the file is empty */
-  c = fgetc(f);
-  if (c==EOF) {
-    return -3;
-  }
-  ungetc(c, f);
 
   /* read magic number. We ignore whitespace and comments before the
      magic, for the benefit of concatenated files in P1-P3 format.
@@ -188,7 +181,8 @@ int gm_read(FILE *f, greymap_t **gmp) {
 
   magic[0] = fgetc_ws(f);
   if (magic[0] == EOF) {
-    return -4;
+    /* files which contain only comments and whitespace count as "empty" */
+    return -3;
   } 
   magic[1] = fgetc(f);
   if (magic[0] == 'P' && magic[1] >= '1' && magic[1] <= '6') {
@@ -245,7 +239,7 @@ static int gm_readbody_pnm(FILE *f, greymap_t **gmp, int magic) {
 	if (b<0) {
 	  goto eof;
 	}
-	GM_UPUT(gm, x, y, b);
+	GM_UPUT(gm, x, y, b ? 0 : 255);
       }
     }
     break;
@@ -488,7 +482,7 @@ static int bmp_forward(FILE *f, int pos) {
 #define TRY_EOF(x) if (x) goto eof
 
 /* read BMP stream after magic number. Return values as for gm_read.
-   We choose to be as permitting as possible, since there are many
+   We choose to be as permissive as possible, since there are many
    programs out there which produce BMP. For instance, ppmtobmp can
    produce codings with anywhere from 1-8 or 24 bits per sample,
    although most specifications only allow 1,4,8,24,32. We can also
@@ -563,7 +557,7 @@ int gm_readbody_bmp(FILE *f, greymap_t **gmp) {
       goto std_error;
     }
     /* NOTE: since we are reading a greymap, we can immediately convert
-       the color table entries to bits. */
+       the color table entries to grey values. */
     for (i=0; i<bmpinfo.ncolors; i++) {
       TRY(bmp_readint(f, bmpinfo.ctbits/8, &c));
       c = ((c>>16) & 0xff) + ((c>>8) & 0xff) + (c & 0xff);
