@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: main.c,v 1.30 2004/06/20 15:30:30 selinger Exp $ */
+/* $Id: main.c,v 1.41 2005/02/22 21:31:23 selinger Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -12,14 +12,18 @@
 #include <math.h>
 
 #include "main.h"
+#include "potracelib.h"
+#include "backend_pdf.h"
 #include "backend_eps.h"
 #include "backend_pgm.h"
 #include "backend_svg.h"
 #include "backend_gimp.h"
 #include "backend_xfig.h"
-#include "curve.h"
-#include "path.h"
+#include "potracelib.h"
+#include "bitmap_io.h"
 #include "bitmap.h"
+#include "platform.h"
+#include "auxiliary.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,14 +33,82 @@
 #define M_PI 3.14159265358979323846
 #endif
 
-#define max(a,b) ((a)>(b) ? (a) : (b))
-#define min(a,b) ((a)<(b) ? (a) : (b))
-
 #define UNDEF ((double)(1e30))   /* a value to represent "undefined" */
 #define INFTY ((double)(1e30))   /* a value to represent +infinity */
 
 struct info info;
 
+#define COL0 "\033[G"  /* reset cursor to column 0 */
+
+/* ---------------------------------------------------------------------- */
+/* callback function for progress bar */
+
+struct simple_progress_s {
+  char name[22];          /* filename for status bar */
+  double dnext;           /* threshold value for next tick */
+};
+typedef struct simple_progress_s simple_progress_t;
+
+/* print a simple progress bar. This is a callback function that is
+   potentially called often; thus, it has been optimized for the
+   typical case, which is when the progress bar does not need updating. */
+static void simple_progress(double d, void *data) {
+  simple_progress_t *p = (simple_progress_t *)data;
+  static char b[] = "========================================";
+  int tick;    /* number of visible tickmarks, 0..40 */
+  int perc;    /* visible percentage, 0..100 */
+
+  /* note: the 0.01 and 0.025 ensure that we always end on 40
+     tickmarks and 100%, despite any rounding errors. The 0.995
+     ensures that tick always increases when d >= p->dnext. */
+  if (d >= p->dnext) {
+    tick = (int) floor(d*40+0.01);
+    perc = (int) floor(d*100+0.025);
+    fprintf(stderr, "%-21s |%-40s| %d%% "COL0"", p->name, b+40-tick, perc);
+    p->dnext = (tick+0.995) / 40.0;
+  }
+}
+
+/* Initialize parameters for simple progress bar. The caller passes an
+   allocated simple_progress_t structure to avoid having to malloc it
+   here and free it later. */
+static inline void init_progress(potrace_progress_t *prog, simple_progress_t *p, const char *filename, int count) {
+  const char *q, *s;
+  int len;
+
+  /* initialize callback function's data */
+  p->dnext = 0;
+
+  if (count != 0) {
+    sprintf(p->name, " (p.%d):", count+1);
+  } else {
+    s = filename;
+    if ((q = strrchr(s, '/')) != NULL) {
+      s = q+1;
+    }
+    len = strlen(s);
+    strncpy(p->name, s, 21);
+    p->name[20] = 0;
+    if (len > 20) {
+      p->name[17] = '.';
+      p->name[18] = '.';
+      p->name[19] = '.';
+    }
+    strcat(p->name, ":");
+  }
+
+  /* initialize progress parameters */
+  prog->callback = &simple_progress;
+  prog->data = (void *)p;
+  prog->min = 0.0;
+  prog->max = 1.0;
+  prog->epsilon = 0.0;
+  
+  /* draw first progress bar */
+  simple_progress(0.0, prog->data);
+  return;
+}
+
 /* ---------------------------------------------------------------------- */
 /* some data structures for option processing */
 
@@ -71,13 +143,13 @@ typedef struct turnpolicy_s turnpolicy_t;
 
 /* names of turn policies */
 static turnpolicy_t turnpolicy[] = {
-  {"black",    POLICY_BLACK},
-  {"white",    POLICY_WHITE},
-  {"left",     POLICY_LEFT},
-  {"right",    POLICY_RIGHT},
-  {"minority", POLICY_MINORITY},
-  {"majority", POLICY_MAJORITY},
-  {"random",   POLICY_RANDOM},
+  {"black",    POTRACE_TURNPOLICY_BLACK},
+  {"white",    POTRACE_TURNPOLICY_WHITE},
+  {"left",     POTRACE_TURNPOLICY_LEFT},
+  {"right",    POTRACE_TURNPOLICY_RIGHT},
+  {"minority", POTRACE_TURNPOLICY_MINORITY},
+  {"majority", POTRACE_TURNPOLICY_MAJORITY},
+  {"random",   POTRACE_TURNPOLICY_RANDOM},
   {NULL, 0},
 };
 
@@ -89,20 +161,22 @@ struct backend_s {
   int pixel;        /* pixel-based backend? */
   int multi;        /* multi-page backend? */
   int (*init_f)(FILE *fout);                 /* initialization function */
-  int (*page_f)(FILE *fout, path_t *plist, imginfo_t *imginfo);
+  int (*page_f)(FILE *fout, potrace_path_t *plist, imginfo_t *imginfo);
                                              /* per-bitmap function */
   int (*term_f)(FILE *fout);                 /* finalization function */
+  int opticurve;    /* opticurve capable (true Bezier curves?) */
 };
 typedef struct backend_s backend_t;  
 
 static backend_t backend[] = {
-  {"eps",        ".eps",      0, 0, 0, NULL,    page_eps,   NULL    },
-  {"postscript", ".ps",       1, 0, 1, init_ps, page_ps,    term_ps },
-  {"ps",         ".ps",       1, 0, 1, init_ps, page_ps,    term_ps },
-  {"svg",        ".svg",      0, 0, 0, NULL,    page_svg,   NULL    },
-  {"pgm",        ".pgm",      0, 1, 1, NULL,    page_pgm,   NULL    },
-  {"gimppath",   ".gimppath", 0, 1, 0, NULL,    page_gimp,  NULL    },
-  {"xfig",       ".fig",      1, 0, 0, NULL,    page_xfig,  NULL    },
+  {"eps",        ".eps",      0, 0, 0, NULL,    page_eps,   NULL,     1},
+  {"postscript", ".ps",       1, 0, 1, init_ps, page_ps,    term_ps,  1},
+  {"ps",         ".ps",       1, 0, 1, init_ps, page_ps,    term_ps,  1},
+  {"pdf",        ".pdf",      0, 0, 1, init_pdf,page_pdf,   term_pdf, 1},
+  {"svg",        ".svg",      0, 0, 0, NULL,    page_svg,   NULL,     1},
+  {"pgm",        ".pgm",      0, 1, 1, NULL,    page_pgm,   NULL,     1},
+  {"gimppath",   ".gimppath", 0, 1, 0, NULL,    page_gimp,  NULL,     1},
+  {"xfig",       ".fig",      1, 0, 0, NULL,    page_xfig,  NULL,     0},
   {NULL, NULL, 0, 0, 0, NULL, NULL, NULL},
 };
 
@@ -134,6 +208,26 @@ static int backend_lookup(char *name, backend_t **bp) {
   }
 }
 
+/* list all available backends by name, in a comma separated list.
+   Assume the cursor starts in column j, and break lines at length
+   linelen. Do not output any trailing punctuation. Return the column
+   the cursor is in. */
+static int backend_list(FILE *fout, int j, int linelen) {
+  int i;
+
+  for (i=0; backend[i].name; i++) {
+    if (j + (int)strlen(backend[i].name) > linelen) {
+      fprintf(fout, "\n");
+      j = 0;
+    }
+    j += fprintf(fout, "%s", backend[i].name);
+    if (backend[i+1].name) {
+      j += fprintf(fout, ", ");
+    }
+  }
+  return j;
+}
+
 /* ---------------------------------------------------------------------- */
 /* some info functions */
 
@@ -164,12 +258,15 @@ static int show_defaults(FILE *f) {
 }
 
 static int usage(FILE *f) {
+  int j;
+
   fprintf(f, "Usage: "POTRACE" [options] [file...]\n");
   fprintf(f, "General options:\n");
   fprintf(f, " -h, --help                 - print this help message and exit\n");
   fprintf(f, " -v, --version              - print version info and exit\n");
   fprintf(f, " -l, --license              - print license info and exit\n");
   fprintf(f, " -V, --show-defaults        - print compiled-in defaults and exit\n");
+  fprintf(f, " --progress                 - show progress bar\n");
   fprintf(f, "Input/output options:\n");
   fprintf(f, " -o, --output <file>        - output to file\n");
   fprintf(f, "Backend selection:\n");
@@ -220,7 +317,9 @@ static int usage(FILE *f) {
   fprintf(f, "Dimensions can have optional units, e.g. 6.5in, 15cm, 100pt.\n");
   fprintf(f, "Default is "DEFAULT_DIM_NAME" (or pixels for pgm and gimppath backends).\n");
   fprintf(f, "Possible input file formats are: pnm (pbm, pgm, ppm), bmp.\n");
-  fprintf(f, "Backends are: eps, postscript, ps, svg, pgm, gimppath, xfig.\n");
+  j = fprintf(f, "Backends are: ");
+  backend_list(f, j, 70);
+  fprintf(f, ".\n");
   
   return 0;
 }
@@ -297,7 +396,7 @@ static void parse_dimensions(char *s, char **endptr, dim_t *dxp, dim_t *dyp) {
   return;
 }
 
-static double double_of_dim(dim_t d, double def) {
+static inline double double_of_dim(dim_t d, double def) {
   if (d.d) {
     return d.x * d.d;
   } else {
@@ -334,12 +433,14 @@ static int parse_color(char *s) {
 #define OPT_GROUP     300
 #define OPT_OPAQUE    301
 #define OPT_FILLCOLOR 302
+#define OPT_PROGRESS  303
 
 static struct option longopts[] = {
   {"help",          0, 0, 'h'},
   {"version",       0, 0, 'v'},
   {"license",       0, 0, 'l'},
   {"show-defaults", 0, 0, 'V'},
+  {"progress",      0, 0, OPT_PROGRESS},
   {"width",         1, 0, 'W'},
   {"height",        1, 0, 'H'},
   {"resolution",    1, 0, 'r'},
@@ -406,23 +507,24 @@ static int dopts(int ac, char *av[]) {
   info.angle = 0;
   info.paperwidth = DEFAULT_PAPERWIDTH;
   info.paperheight = DEFAULT_PAPERHEIGHT;
-  info.turdsize = 2;
   info.unit = 10;
   info.compress = 1;
   info.pslevel = 2;
   info.color = 0x000000;
-  info.turnpolicy = POLICY_MINORITY;
   info.gamma = 2.2;
-  info.opticurve = 1;
+  info.param = potrace_param_default();
+  if (!info.param) {
+    fprintf(stderr, ""POTRACE": %s\n", strerror(errno));
+    exit(1);
+  }
   info.longcoding = 0;
-  info.alphamax = 1.0;
-  info.opttolerance = 0.2;
   info.outfile = NULL;
   info.blacklevel = 0.5;
   info.invert = 0;
   info.opaque = 0;
   info.group = 0;
   info.fillcolor = 0xffffff;
+  info.progress = 0;
 
   while ((c = getopt_long(ac, av, shortopts, longopts, NULL)) != -1) {
     switch (c) {
@@ -432,19 +534,23 @@ static int dopts(int ac, char *av[]) {
       exit(0);
       break;
     case 'v':
-      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2004 Peter Selinger.\n");
+      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2005 Peter Selinger.\n");
+      fprintf(stdout, "Library version: %s\n", potrace_version());
       exit(0);
       break;
     case 'l':
-      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2004 Peter Selinger.\n\n");
+      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2005 Peter Selinger.\n\n");
       license(stdout);
       exit(0);
       break;
     case 'V':
-      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2004 Peter Selinger.\n");
+      fprintf(stdout, ""POTRACE" "VERSION". Copyright (C) 2001-2005 Peter Selinger.\n");
       show_defaults(stdout);
       exit(0);
       break;
+    case OPT_PROGRESS:
+      info.progress = 1;
+      break;
     case 'W':
       info.width_d = parse_dimension(optarg, &p);
       if (*p) {
@@ -579,7 +685,7 @@ static int dopts(int ac, char *av[]) {
       exit(1);
       break;
     case 't':
-      info.turdsize = atoi(optarg);
+      info.param->turdsize = atoi(optarg);
       break;
     case 'u':
       info.unit = strtod(optarg, &p);
@@ -627,16 +733,7 @@ static int dopts(int ac, char *av[]) {
 	  fprintf(stderr, ""POTRACE": ambiguous backend -- %s\n", optarg);
 	}
 	j = fprintf(stderr, "Use one of: ");
-	for (i=0; backend[i].name; i++) {
-	  if (j + strlen(backend[i].name) > 75) {
-	    fprintf(stderr, "\n");
-	    j = 0;
-	  }
-	  j += fprintf(stderr, "%s", backend[i].name);
-	  if (backend[i+1].name) {
-	    j += fprintf(stderr, ", ");
-	  }
-	}
+	backend_list(stderr, j, 70);
 	fprintf(stderr, ".\n");
 	exit(1);
       }
@@ -673,7 +770,7 @@ static int dopts(int ac, char *av[]) {
 	}
       }
       if (matches == 1) {
-	info.turnpolicy = turnpolicy[bestmatch].n;
+	info.param->turnpolicy = turnpolicy[bestmatch].n;
 	break;
       }
       if (matches == 0) {
@@ -696,20 +793,20 @@ static int dopts(int ac, char *av[]) {
       info.gamma = atof(optarg);
       break;
     case 'n':
-      info.opticurve = 0;
+      info.param->opticurve = 0;
       break;
     case 'q':
       info.longcoding = 1;
       break;
     case 'a':
-      info.alphamax = strtod(optarg, &p);
+      info.param->alphamax = strtod(optarg, &p);
       if (*p) {
 	fprintf(stderr, ""POTRACE": invalid alphamax -- %s\n", optarg);
 	exit(1);
       }
       break;
     case 'O':
-      info.opttolerance = strtod(optarg, &p);
+      info.param->opttolerance = strtod(optarg, &p);
       if (*p) {
 	fprintf(stderr, ""POTRACE": invalid opttolerance -- %s\n", optarg);
 	exit(1);
@@ -983,11 +1080,12 @@ static char *make_outfilename(char *infile, char *ext) {
 
 static void process_file(backend_t *b, const char *infile, const char *outfile, FILE *fin, FILE *fout) { 
   int r; 
-  bitmap_t *bm = NULL; 
+  potrace_bitmap_t *bm = NULL; 
   imginfo_t imginfo;
-  path_t *plist;
   int eof_flag = 0;  /* to indicate premature eof */
   int count;         /* number of bitmaps successfully processed, this file */
+  potrace_state_t *st;
+  simple_progress_t progress_data;
 
   for (count=0; ; count++) {
     /* read a bitmap */
@@ -1019,13 +1117,20 @@ static void process_file(backend_t *b, const char *infile, const char *outfile,
       break;
     }
 
+    /* prepare progress bar, if requested */
+    if (info.progress) {
+      init_progress(&info.param->progress, &progress_data, infile, count);
+    } else {
+      info.param->progress.callback = NULL;
+    }
+
     if (info.invert) {
       bm_invert(bm);
     }
 
     /* process the image */
-    r = bm_to_pathlist(bm, &plist);
-    if (r) {
+    st = potrace_trace(info.param, bm);
+    if (!st || st->status != POTRACE_STATUS_OK) {
       fprintf(stderr, ""POTRACE": %s: %s\n", infile, strerror(errno));
       exit(2);
     }
@@ -1037,20 +1142,17 @@ static void process_file(backend_t *b, const char *infile, const char *outfile,
 
     bm_free(bm);
 
-    r = process_path(plist);
-    if (r) {
-      fprintf(stderr, ""POTRACE": %s: %s\n", infile, strerror(errno));
-      exit(2);
-    }
-
-    r = b->page_f(fout, plist, &imginfo);
+    r = b->page_f(fout, st->plist, &imginfo);
     if (r) {
       fprintf(stderr, ""POTRACE": %s: %s\n", outfile, strerror(errno));
       exit(2);
     }
 
-    pathlist_free(plist);
-    plist = NULL;
+    potrace_state_free(st);
+
+    if (info.progress) {
+      fprintf(stderr, "\n");
+    }
 
     if (eof_flag || !b->multi) {
       return;
@@ -1078,6 +1180,12 @@ int main(int ac, char *av[]) {
     exit(1);
   }
 
+  /* fix some parameters */
+  /* if backend cannot handle opticurve, disable it */
+  if (b->opticurve == 0) {
+    info.param->opticurve = 0;
+  }
+
   /* there are several ways to call us:
      potrace                     -- stdin to stdout
      potrace -o outfile          -- stdin to outfile
