@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: main.h,v 1.19 2004/01/15 05:07:22 selinger Exp $ */
+/* $Id: main.h,v 1.22 2005/02/22 21:31:23 selinger Exp $ */
 
 #ifndef MAIN_H
 #define MAIN_H
@@ -11,13 +11,7 @@
 #include "config.h"
 #endif
 
-#define POLICY_BLACK 0
-#define POLICY_WHITE 1
-#define POLICY_LEFT 2
-#define POLICY_RIGHT 3
-#define POLICY_MINORITY 4
-#define POLICY_MAJORITY 5
-#define POLICY_RANDOM 6
+#include "potracelib.h"
 
 /* structure to hold a tilted rectangle */
 struct rect_s {
@@ -63,6 +57,7 @@ struct backend_s;
 /* structure to hold command line options */
 struct info {
   struct backend_s *backend;  /* type of backend (eps,ps,pgm etc) */
+  potrace_param_t *param;  /* tracing parameters, see potracelib.h */
   int debug;         /* type of output (0-2) (for BACKEND_PS/EPS only) */
   dim_t width_d;     /* desired width of image */
   dim_t height_d;    /* desired height of image */
@@ -74,18 +69,13 @@ struct info {
   dim_t lmar_d, rmar_d, tmar_d, bmar_d;   /* margins */
   double angle;      /* rotate by this many degrees */
   int paperwidth, paperheight;  /* paper size for ps backend (in pt) */
-  int turdsize;      /* area of largest path to be ignored */
   double unit;       /* granularity of output grid */
   int compress;      /* apply compression? */
   int pslevel;       /* postscript level to use: affects only compression */
   int color;         /* rgb color code 0xrrggbb: line color */
   int fillcolor;     /* rgb color code 0xrrggbb: fill color */
-  int turnpolicy;    /* how to resolve ambiguous turns in path decomposition */
   double gamma;      /* gamma value for pgm backend */
-  int opticurve;     /* use curve optimization */
   int longcoding;    /* do not optimize for file size? */
-  double alphamax;   /* corner threshold parameter */
-  double opttolerance; /* opticurve tolerance */
   char *outfile;     /* output filename, if given */
   char **infiles;    /* array of input filenames */
   int infilecount;   /* number of input filenames */
@@ -93,9 +83,11 @@ struct info {
   int invert;        /* invert bitmap? */
   int opaque;        /* paint white shapes opaquely? */
   int group;         /* group paths together? */
+  int progress;      /* should we display a progress bar? */
 };
+typedef struct info info_t;
 
-extern struct info info;
+extern info_t info;
 
 /* structure to hold per-image information, set e.g. by calc_dimensions */
 struct imginfo_s {
