@@ -1,6 +1,6 @@
-/* Copyright (C) 2001-2004 Peter Selinger.
+/* Copyright (C) 2001-2005 Peter Selinger.
    This file is part of potrace. It is free software and it is covered
-   by the GNU general public license. See the file COPYING for details. */
+   by the GNU General Public License. See the file COPYING for details. */
 
 /* mkbitmap.c: a standalone program for converting greymaps to bitmaps
    while optionally applying the following enhancements: highpass
@@ -14,7 +14,8 @@
 #include <getopt.h>
 
 #include "greymap.h"
-#include "bitmap.h"
+#include "bitmap_io.h"
+#include "platform.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -134,7 +135,7 @@ static int highpass(greymap_t *gm, double lambda) {
 
 /* scale greymap by factor s, using linear interpolation. If
    bilevel=0, return a pointer to a greymap_t. If bilevel=1, return a
-   pointer to a bitmap_t and use cutoff threshold c (0=black,
+   pointer to a potrace_bitmap_t and use cutoff threshold c (0=black,
    1=white).  On error, return NULL with errno set. */
 
 static void *interpolate_linear(greymap_t *gm, int s, int bilevel, double c) {
@@ -145,7 +146,7 @@ static void *interpolate_linear(greymap_t *gm, int s, int bilevel, double c) {
   int w, h;
   double p0, p1;
   greymap_t *gm_out = NULL;
-  bitmap_t *bm_out = NULL;
+  potrace_bitmap_t *bm_out = NULL;
 
   w = gm->w;
   h = gm->h;
@@ -227,7 +228,7 @@ static void *interpolate_cubic(greymap_t *gm, int s, int bilevel, double c) {
   int k, l, i, j, x, y;
   double c1 = 0;
   greymap_t *gm_out = NULL;
-  bitmap_t *bm_out = NULL;
+  potrace_bitmap_t *bm_out = NULL;
 
   SAFE_MALLOC(poly, s, double4);
   SAFE_MALLOC(window, s, double4);
@@ -333,7 +334,7 @@ static void *interpolate_cubic(greymap_t *gm, int s, int bilevel, double c) {
 static void process_file(FILE *fin, FILE *fout, char *infile, char *outfile) {
   int r;
   greymap_t *gm;
-  bitmap_t *bm;
+  potrace_bitmap_t *bm;
   void *sm;
   int x, y;
   int count;
@@ -394,7 +395,7 @@ static void process_file(FILE *fin, FILE *fout, char *infile, char *outfile) {
     gm_free(gm);
     
     if (info.bilevel) {
-      bm = (bitmap_t *)sm;
+      bm = (potrace_bitmap_t *)sm;
       bm_writepbm(fout, bm);
       bm_free(bm);
     } else {
@@ -501,11 +502,11 @@ static void dopts(int ac, char *av[]) {
       exit(0);
       break;
     case 'v':
-      fprintf(stdout, ""MKBITMAP" "VERSION". Copyright (C) 2001-2004 Peter Selinger.\n");
+      fprintf(stdout, ""MKBITMAP" "VERSION". Copyright (C) 2001-2005 Peter Selinger.\n");
       exit(0);
       break;
     case 'l':
-      fprintf(stdout, ""MKBITMAP" "VERSION". Copyright (C) 2001-2004 Peter Selinger.\n\n");
+      fprintf(stdout, ""MKBITMAP" "VERSION". Copyright (C) 2001-2005 Peter Selinger.\n\n");
       license(stdout);
       exit(0);
       break;
